from linguaf import readability as r


def test_flesh_reading_ease():
    fre_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 25),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 33)
    ]

    fre_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 44),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 79)
    ]

    for d in fre_data_ru:
        fre = d[1]
        res = int(r.flesch_reading_ease(d[0], 'ru', False))

        assert fre == res

    for d in fre_data_en:
        fre = d[1]
        res = int(r.flesch_reading_ease(d[0], 'en', False))

        assert fre == res


def test_flesh_kincaid_grade():
    read_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 13),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 12)
    ]

    read_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 13),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 6)
    ]

    for d in read_data_ru:
        score = d[1]
        res = int(r.flesch_kincaid_grade(d[0], 'ru', False))

        assert score == res

    for d in read_data_en:
        score = d[1]
        res = int(r.flesch_kincaid_grade(d[0], 'en', False))

        assert score == res


def test_automated_readability_index():
    read_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 18),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 16)
    ]

    read_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 15),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 10)
    ]

    for d in read_data_ru:
        score = d[1]
        res = int(r.automated_readability_index(d[0], 'ru', False))

        assert score == res

    for d in read_data_en:
        score = d[1]
        res = int(r.automated_readability_index(d[0], 'en', False))

        assert score == res


def test_automated_readability_index_simple():
    read_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 76),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 73)
    ]

    read_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 72),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 61)
    ]

    for d in read_data_ru:
        score = d[1]
        res = int(r.automated_readability_index_simple(d[0], 'ru', False))

        assert score == res

    for d in read_data_en:
        score = d[1]
        res = int(r.automated_readability_index_simple(d[0], 'en', False))

        assert score == res


def test_coleman_readability():
    read_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 20),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 24)
    ]

    read_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 69),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 80)
    ]

    for d in read_data_ru:
        score = d[1]
        res = int(r.coleman_readability(d[0], 'ru', False))

        assert score == res

    for d in read_data_en:
        score = d[1]
        res = int(r.coleman_readability(d[0], 'en', False))

        assert score == res


def test_easy_listening():
    read_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 2),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 2)
    ]

    read_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 5),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 2)
    ]

    for d in read_data_ru:
        score = d[1]
        res = int(r.easy_listening(d[0], 'ru', False))

        assert score == res

    for d in read_data_en:
        score = d[1]
        res = int(r.easy_listening(d[0], 'en', False))

        assert score == res


from linguaf import lexical_diversity as ld


def test_lexical_density():
    ld_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 70),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 70)
    ]

    ld_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 59),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 65)
    ]

    for d in ld_data_ru:
        cnt = d[1]
        res = int(ld.lexical_density(d[0], 'ru'))

        assert cnt == res

    for d in ld_data_en:
        cnt = d[1]
        res = int(ld.lexical_density(d[0], 'en'))

        assert cnt == res


def test_type_token_ratio():
    ttr_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 85),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 100)
    ]

    ttr_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 68),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 85)
    ]

    for d in ttr_data_ru:
        cnt = d[1]
        res = int(ld.type_token_ratio(d[0], 'ru', True))

        assert cnt == res

    for d in ttr_data_en:
        cnt = d[1]
        res = int(ld.type_token_ratio(d[0], 'en', True))

        assert cnt == res


def test_log_type_token_ratio():
    ttr_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 95),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 100)
    ]

    ttr_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 89),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 95)
    ]

    for d in ttr_data_ru:
        cnt = d[1]
        res = int(ld.log_type_token_ratio(d[0], 'ru', True))

        assert cnt == res

    for d in ttr_data_en:
        cnt = d[1]
        res = int(ld.log_type_token_ratio(d[0], 'en', True))

        assert cnt == res


def test_summer_index():
    ttr_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 96),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 100)
    ]

    ttr_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 90),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 96)
    ]

    for d in ttr_data_ru:
        cnt = d[1]
        res = int(ld.summer_index(d[0], 'ru', True))

        assert cnt == res

    for d in ttr_data_en:
        cnt = d[1]
        res = int(ld.summer_index(d[0], 'en', True))

        assert cnt == res


def test_root_type_token_ratio():
    ttr_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 497),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 538)
    ]

    ttr_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 388),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 591)
    ]

    for d in ttr_data_ru:
        cnt = d[1]
        res = int(ld.root_type_token_ratio(d[0], 'ru', True))

        assert cnt == res

    for d in ttr_data_en:
        cnt = d[1]
        res = int(ld.root_type_token_ratio(d[0], 'en', True))

        assert cnt == res



from linguaf import lexical_complexity as lc


def test_mean_dependency_distance():
    ttr_data_ru = [
        ([
             "Дивергенция — дифференциальный оператор, отображающий векторное поле на скалярное.",
             "То есть, в результате применения к векторному полю операции дифференцирования получается скалярное поле.",
             "Она определяет (для каждой точки), «насколько расходится входящее и исходящее из малой окрестности данной точки поле»",
             "Точнее, насколько расходятся входящий и исходящий потоки."
         ], 2),
        ([
             "«Лунтик и его друзья» — российский мультсериал, ориентированный на семейную и детскую аудиторию.",
             "Транслируется на телевидении с 1 сентября 2006 года по настоящее время.",
             "Ключевой темой стали приключения маленького пушистого существа Лунтика — космического пришельца, который родился на Луне."
         ], 2)
    ]

    ttr_data_en = [
        ([
             "In vector calculus, divergence is a vector operator that operates on a vector field, producing a scalar field giving the quantity of the vector field's source at each point.",
             "More technically, the divergence represents the volume density of the outward flux of a vector field from an infinitesimal volume around a given point."
         ], 2),
        ([
             "Luntik and his friends (Russian: Лунтик и его друзья) or simply Luntik (Russian: Лунтик), in its English version better known as Moonzy, is a Russian animated series for children.",
             "The title character is a purple furry alien named Luntik/Moonzy.",
             "In the first episode Moonzy is shown hatching from an egg on the moon and then falling to Earth.",
             "All of the following series take place at a forest glade near a pond where Moonzy finds a new home."
         ], 2)
    ]

    for d in ttr_data_ru:
        cnt = d[1]
        res = int(lc.mean_dependency_distance(d[0], 'ru'))

        assert cnt == res

    for d in ttr_data_en:
        cnt = d[1]
        res = int(lc.mean_dependency_distance(d[0], 'en'))

        assert cnt == res



from linguaf import descriptive_statistics as ds


def test_sentence_count():
    ru_sentence_cnt = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки.", "Пользуйтесь на здоровье!"], 3),
        (["Пока"], 1),
        (["Привет...это я...", "Как дела? Хорошо?"], 4)
    ]

    en_sentence_cnt = [
        (["Hello, I'm Aleksandr! I'm the creator of this library.", "Enjoy using it!"], 3),
        (["Bye"], 1),
        (["Hello...it's me...", "How are you? Good!"], 4)
    ]

    for i in range(len(ru_sentence_cnt)):
        v = ru_sentence_cnt[i][1]
        res = ds.sentence_count(ru_sentence_cnt[i][0])
        assert v == res

    for i in range(len(en_sentence_cnt)):
        v = en_sentence_cnt[i][1]
        res = ds.sentence_count(en_sentence_cnt[i][0])
        assert v == res


def test_char_count():
    ru_char_cnt = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки.", "Пользуйтесь на здоровье!"], 82, 73),
        (["Пока"], 4, 4),
        (["Привет...это я...", "Как дела? Хорошо?"], 34, 31)
    ]

    en_char_cnt = [
        (["Hello, I'm Aleksandr! I'm the creator of this library.", "Enjoy using it!"], 69, 59),
        (["Bye"], 3, 3),
        (["Hello...it's me...", "How are you? Good!"], 36, 32)
    ]

    for i in range(len(ru_char_cnt)):
        spaces, no_spaces = ru_char_cnt[i][1], ru_char_cnt[i][2]
        spaces_res = ds.char_count(ru_char_cnt[i][0], ignore_spaces=False)
        no_spaces_res = ds.char_count(ru_char_cnt[i][0], ignore_spaces=True)

        assert spaces == spaces_res
        assert no_spaces == no_spaces_res

    for i in range(len(en_char_cnt)):
        spaces, no_spaces = en_char_cnt[i][1], en_char_cnt[i][2]
        spaces_res = ds.char_count(en_char_cnt[i][0], ignore_spaces=False)
        no_spaces_res = ds.char_count(en_char_cnt[i][0], ignore_spaces=True)

        assert spaces == spaces_res
        assert no_spaces == no_spaces_res


def test_tokenize_stopwords():
    ru_tokens = [
        ("Привет, меня зовут Александр! Я создатель этой библиотеки.",
         11,
         ['Привет', ',', 'меня', 'зовут', 'Александр', '!', 'Я', 'создатель', 'этой', 'библиотеки', '.']),
        ("Пользуйтесь на здоровье!",
         4,
         ['Пользуйтесь', 'на', 'здоровье', '!']),
        ("Пока",
         1,
         ['Пока']),
        ("Привет...это я...",
         5,
         ['Привет', '...', 'это', 'я', '...']),
        ("Как дела? Хорошо?",
         5,
         ['Как', 'дела', '?', 'Хорошо', '?'])
    ]

    en_tokens = [
        ("Hello, I'm Aleksandr! I'm the creator of this library.",
         14,
         ['Hello', ',', 'I', "'m", 'Aleksandr', '!', 'I', "'m", 'the', 'creator', 'of', 'this', 'library', '.']),
        ("Enjoy using it!",
         4,
         ['Enjoy', 'using', 'it', '!']),
        ("Bye",
         1,
         ["Bye"]),
        ("Hello...it's me...",
         6,
         ['Hello', '...', 'it', "'s", 'me', '...']),
        ("How are you? Good!",
         6,
         ['How', 'are', 'you', '?', 'Good', '!'])
    ]

    for i in range(len(ru_tokens)):
        cnt = ru_tokens[i][1]
        true_tokens = ru_tokens[i][2]

        tokens = ds.tokenize(text=ru_tokens[i][0], lang='ru', remove_stopwords=False)

        assert len(tokens) == cnt
        assert tokens == true_tokens

    for i in range(len(en_tokens)):
        cnt = en_tokens[i][1]
        true_tokens = en_tokens[i][2]

        tokens = ds.tokenize(text=en_tokens[i][0], lang='en', remove_stopwords=False)

        assert len(tokens) == cnt
        assert tokens == true_tokens


def test_tokenize_remove_stopwords():
    ru_tokens = [
        ("Привет, меня зовут Александр! Впрочем, это другая история.",
         10,
         ['Привет', ',', 'зовут', 'Александр', '!', ',', 'это', 'другая', 'история', '.']),
        ("Пользуйтесь на здоровье!",
         3,
         ['Пользуйтесь', 'здоровье', '!']),
        ("Пока",
         1,
         ['Пока']),
        ("Привет...это я...",
         4,
         ['Привет', '...', 'это', '...']),
        ("Как дела? Хорошо?",
         3,
         ['дела', '?', '?'])
    ]

    en_tokens = [
        ("Hello, I'm Aleksandr! I'm the creator of this library.",
         9,
         ['Hello', ',', "'m", 'Aleksandr', '!', "'m", 'creator', 'library', '.']),
        ("Enjoy using it!",
         3,
         ['Enjoy', 'using', '!']),
        ("Bye",
         1,
         ["Bye"]),
        ("Hello...it's me...",
         4,
         ['Hello', '...', "'s", '...']),
        ("How are you? Good!",
         3,
         ['?', 'Good', '!'])
    ]

    for i in range(len(ru_tokens)):
        cnt = ru_tokens[i][1]
        true_tokens = ru_tokens[i][2]

        tokens = ds.tokenize(text=ru_tokens[i][0], lang='ru', remove_stopwords=True)

        assert len(tokens) == cnt
        assert tokens == true_tokens

    for i in range(len(en_tokens)):
        cnt = en_tokens[i][1]
        true_tokens = en_tokens[i][2]

        tokens = ds.tokenize(text=en_tokens[i][0], lang='en', remove_stopwords=True)

        assert len(tokens) == cnt
        assert tokens == true_tokens


def test_get_words():
    # stopwords removal tested in test_tokenize_remove_stopwords
    ru_words = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки."],
         8,
         ['Привет', 'меня', 'зовут', 'Александр', 'Я', 'создатель', 'этой', 'библиотеки']),
        (["Пользуйтесь на здоровье!"],
         3,
         ['Пользуйтесь', 'на', 'здоровье']),
        (["Пока"],
         1,
         ['Пока']),
        (["Привет...это я..."],
         3,
         ['Привет', 'это', 'я']),
        (["Как дела? Хорошо?"],
         3,
         ['Как', 'дела', 'Хорошо'])
    ]

    en_words = [
        (["Hello, I'm Aleksandr! I'm the creator of this library."],
         11,
         ['Hello', 'I', "'m", 'Aleksandr', 'I', "'m", 'the', 'creator', 'of', 'this', 'library']),
        (["Enjoy using it!"],
         3,
         ['Enjoy', 'using', 'it']),
        (["Bye"],
         1,
         ["Bye"]),
        (["Hello...it's me..."],
         4,
         ['Hello', 'it', "'s", 'me']),
        (["How are you? Good!"],
         4,
         ['How', 'are', 'you', 'Good'])
    ]

    for i in range(len(ru_words)):
        cnt = ru_words[i][1]
        true_words = ru_words[i][2]

        words = ds.get_words(documents=ru_words[i][0], lang='ru', remove_stopwords=False)

        assert len(words) == cnt
        assert words == true_words

    for i in range(len(en_words)):
        cnt = en_words[i][1]
        true_words = en_words[i][2]

        words = ds.get_words(documents=en_words[i][0], lang='en', remove_stopwords=False)

        assert len(words) == cnt
        assert words == true_words


def test_remove_punctuation():
    punctuation_data = [
        ("Hello, how are you?", "Hello how are you"),
        ("Привет, как дела?", "Привет как дела"),
        ("Hi...", "Hi"),
        ("Привет...", "Привет"),
    ]
    for i in range(len(punctuation_data)):
        removed = ds.remove_punctuation(punctuation_data[i][0])
        assert removed == punctuation_data[i][1]


def test_letter_count():
    letter_count_data = [
        (["Привет, как дела?", "Что делаешь?"], 23, 26),
        (["Hello, how are you?", "What are you doing?"], 29, 35),
    ]

    for d in letter_count_data:
        ignore_spaces_cnt = d[1]
        with_spaces_cnt = d[2]
        ignore_spaces_result = ds.letter_count(d[0], True)
        with_spaces_result = ds.letter_count(d[0], False)

        assert ignore_spaces_result == ignore_spaces_cnt
        assert with_spaces_result == with_spaces_cnt


def test_avg_sentence_length():
    avg_sentence_len_data = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки.", "Пользуйтесь на здоровье!"], 24, 27),
        (["Пока"], 4, 4),
        (["Привет...это я...", "Как дела? Хорошо?"], 7, 8),
        (["Hello, I'm Aleksandr! I'am the creator of this library.", "Enjoy using it!"], 20, 23),
        (["Bye"], 3, 3),
        (["Hello...it's me...", "How are you? Good!"], 8, 9)
    ]

    for d in avg_sentence_len_data:
        ignore_spaces_len = d[1]
        with_spaces_len = d[2]

        result_ignore_spaces_len = int(ds.avg_sentence_length(d[0], True))
        result_with_spaces_len = int(ds.avg_sentence_length(d[0], False))

        assert with_spaces_len == result_with_spaces_len
        assert ignore_spaces_len == result_ignore_spaces_len


def test_syllable_count():
    syllable_cnt_ru = [
        (["Ротор", "Дивергенция", "Матожидание"], 11)
    ]
    syllable_cnt_en = [
        (["Curl", "Divergence", "Expectation"], 8)
    ]

    for d in syllable_cnt_ru:
        cnt = d[1]
        res = ds.syllable_count(d[0], 'ru')

        assert res == cnt

    for d in syllable_cnt_en:
        cnt = d[1]
        res = ds.syllable_count(d[0], 'en')

    assert res == cnt


def test_avg_syllable_per_word():
    avg_syllables_per_word_ru = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки.", "Пользуйтесь на здоровье!"], 2),
        (["Пока"], 2),
        (["Привет...это я...", "Как дела? Хорошо?"], 1)
    ]

    avg_syllables_per_word_en = [
        (["Hello, I'm Aleksandr! I'am the creator of this library.", "Enjoy using it!"], 1),
        (["Bye"], 1),
        (["Hello...it's me...", "How are you? Good!"], 1)
    ]

    for d in avg_syllables_per_word_ru:
        cnt = d[1]
        res = int(ds.avg_syllable_per_word(d[0], 'ru'))

        assert cnt == res

    for d in avg_syllables_per_word_en:
        cnt = d[1]
        res = int(ds.avg_syllable_per_word(d[0], 'en'))

        assert cnt == res


def test_get_lexical_items():
    lexical_items_ru_data = [
        (["Привет, меня зовут Александр! Я создатель этой библиотеки.", "Пользуйтесь на здоровье!"],
         [('Привет', 'NOUN'), ('зовут', 'VERB'), ('Александр', 'NOUN'), ('создатель', 'NOUN'), ('этой', 'ADJF'),
          ('библиотеки', 'NOUN'), ('Пользуйтесь', 'VERB'), ('здоровье', 'NOUN')]),
        (["Пока"], [('Пока', 'ADVB')]),
        (["Привет...это я...", "Как дела? Хорошо?"], [('Привет', 'NOUN'), ('дела', 'NOUN'), ('Хорошо', 'ADVB')])
    ]

    lexical_items_en_data = [
        (["Hello, I'm Aleksandr! I'm the creator of this library.", "Enjoy using it!"],
         [('Hello', 'NNP'), ("'m", 'VBP'), ('Aleksandr', 'JJ'), ("'m", 'VBP'), ('creator', 'NN'), ('library', 'NN'),
          ('Enjoy', 'NNP'), ('using', 'VBG')]),
        (["Bye"], [('Bye', 'NN')]),
        (["Hello...it's me...", "How are you? Good!"],
         [('Hello', 'NNP'), ("'s", 'VBZ'), ('are', 'VBP'), ('Good', 'JJ')])
    ]

    for d in lexical_items_ru_data:
        lex_items = d[1]
        result = ds.get_lexical_items(d[0], False, 'ru')

        assert result == lex_items

    for d in lexical_items_en_data:
        lex_items = d[1]
        result = ds.get_lexical_items(d[0], False, 'en')

        assert result == lex_items



import json

SUPPORTED_LANGS = ['en', 'ru']


def __load_json(filepath):
    data = None
    with open(filepath) as f:
        data = json.load(f)
    return data


def __check_documents_param(param):
    if type(param) != list:
        raise TypeError(f"The documents parameter has to be list. Now: {type(param)}")
    for d in param:
        if type(d) != str:
            raise TypeError(f"The documents list should contain strings(str). Now: {type(d)}")


def __check_words_param(param):
    if type(param) != list:
        raise TypeError(f"The words parameter has to be list. Now: {type(param)}")
    for d in param:
        if type(d) != str:
            raise TypeError(f"The words list should contain strings(str). Now: {type(d)}")

def __check_text_param(param):
    if type(param) != str:
        raise TypeError(f"The text should be a string(str). Now: {type(param)}")


def __check_bool_param(param):
    if type(param) != bool:
        raise TypeError(f"The given parameter has to be of type bool. Now: {type(param)}")


def __check_lang_param(param):
    if type(param) != str:
        raise TypeError(f"The lang parameter has to be of type str. Now: {type(param)}")
    if param not in SUPPORTED_LANGS:
        raise ValueError(f"The given language isn't supported. The supported ones are: {SUPPORTED_LANGS}")


from linguaf.descriptive_statistics import get_sentences
from linguaf import __check_documents_param, __check_lang_param
from natasha import Segmenter, NewsSyntaxParser, Doc, NewsEmbedding
import spacy

spacy.cli.download('en_core_web_sm')  # required for english language


def mean_dependency_distance(documents: list, lang: str = 'en') -> float:
    """Calculates Mean Dependency Distance score over a list of documents
    The higher the score the more complex are the sentences.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    """
    __check_documents_param(documents)
    __check_lang_param(lang)

    mdds = list()
    sentences = get_sentences(documents)

    for text in sentences:
        dd = 0
        if lang == 'ru':
            segmenter = Segmenter()
            emb = NewsEmbedding()
            syntax_parser = NewsSyntaxParser(emb)
            doc = Doc(text)
            doc.segment(segmenter)
            doc.parse_syntax(syntax_parser)
            for t in doc.tokens:
                dd += abs(int(t.head_id.split('_')[1]) - int(t.id.split('_')[1]))
            mdd = dd/(len(doc.tokens) - 1)
        elif lang == 'en':
            nlp = spacy.load("en_core_web_sm")
            doc = nlp(text)
            for token in doc:
                dd += abs(token.head.i - token.i)
            mdd = dd/(len(doc) - 1)
        mdds.append(mdd)

    return sum(mdds)/len(mdds)



import os
import re
import pathlib
import string
import pyphen
import pymorphy2
from nltk import word_tokenize, pos_tag
import collections
from linguaf import SUPPORTED_LANGS, __load_json, __check_bool_param, __check_documents_param, __check_lang_param, \
    __check_text_param, __check_words_param


PUNCTUATION = r"""!"#$%&'()*+,-./:;<=>?@[\]^_`{|}~—«»"""
STOPWORDS = dict()


for language in SUPPORTED_LANGS:
    STOPWORDS[language] = __load_json(
        os.path.join(pathlib.Path(__file__).parent.absolute(),
                     "resources",
                     "stopwords",
                     f"{language}.json")
    )


def remove_punctuation(text: str) -> str:
    """Remove punctuation from text string.

    Keyword arguments:
    text -- the string from which the punctuation is removed
    """
    __check_text_param(text)

    return ''.join(ch for ch in text if ch not in PUNCTUATION)


def remove_digits(text: str) -> str:
    """Remove digits from text string.

    Keyword arguments:
    text -- the string from which the digits are removed
    """
    __check_text_param(text)

    return ''.join(ch for ch in text if ch not in string.digits)


def char_count(documents: list, ignore_spaces: bool = True) -> int:
    """Count characters in a list of documents.

    Keyword arguments:
    documents -- list of textual documents.
    ignore_spaces -- boolean flag that shows if we should ignore spaces
    """
    __check_documents_param(documents)
    __check_bool_param(ignore_spaces)

    text = str()
    for doc in documents:
        text += doc
    if ignore_spaces:
        text = text.replace(" ", "")
    return len(text)


def letter_count(documents: list, ignore_spaces: bool = True, ignore_digits: bool = True) -> int:
    """Count letters in a list of documents

    Keyword arguments:
    documents -- list of textual documents.
    ignore_spaces -- boolean flag that shows if we should ignore spaces
    ignore_digits -- boolean flag that shows if we should ignore digits
    """
    __check_documents_param(documents)
    __check_bool_param(ignore_spaces)
    __check_bool_param(ignore_digits)

    text = str()
    for doc in documents:
        text += doc
    if ignore_spaces:
        text = text.replace(" ", "")
    if ignore_digits:
        text = remove_digits(text)
    return len(remove_punctuation(text))


def punctuation_count(documents: list) -> int:
    """Count number of punctuation characters in a list of textual documents

    Keyword arguments:
    documents -- the list of textual documents.
    """
    __check_documents_param(documents)

    char_cnt = char_count(documents, ignore_spaces=True)
    char_wo_punctuation = letter_count(documents, ignore_spaces=True, ignore_digits=False)
    return char_cnt - char_wo_punctuation


def digit_count(documents: list) -> int:
    """Count number of digits in a list of textual documents

    Keyword arguments:
    documents -- the list of textual documents.
    """
    __check_documents_param(documents)
    letters_w_digits = letter_count(documents, ignore_spaces=False, ignore_digits=False)
    letters_wo_digits = letter_count(documents, ignore_spaces=False, ignore_digits=True)
    return letters_w_digits - letters_wo_digits


def syllable_count(words: list, lang: str = 'en') -> int:
    """Count number of syllables in a list of words

    Keyword arguments:
    words -- the list of words
    lang -- language of the words
    """
    __check_words_param(words)
    syl_count = 0
    dic = pyphen.Pyphen(lang=lang)  # TODO: match language
    for word in words:
        syl_count += len(dic.inserted(word).split('-'))
    return syl_count


def number_of_n_syllable_words(words: list, lang: str = 'en', n: tuple = (1, 2)) -> int:
    """Count number of words with x <= n < y syllable words in a list of words

    Keyword arguments:
    words -- the list of words
    lang -- language of the words
    n -- tuple: (x, y)
    """
    __check_words_param(words)
    __check_lang_param(lang)
    if n[0] < 1 or n[1] <= n[0]:
        raise ValueError(f"The given n parameter isn't correct: {n}. n=tuple(x,y), x>0, y>x.")

    count = 0
    dic = pyphen.Pyphen(lang=lang)  # TODO: match language
    for word in words:
        syl_cnt = len(dic.inserted(word).split('-'))
        for i in n:
            if syl_cnt == i:
                count += 1
    return count


def get_words(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> list:
    """Create list of words based on a list of textual documents.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = list()

    for doc in documents:
        tokens = tokenize(doc, remove_stopwords=remove_stopwords, lang=lang)
        # add retrieved tokens from a question to a global words list
        words += [t for t in tokens if len(remove_punctuation(t)) > 0]
    if len(words) == 0:
        raise Exception(f"No words found in the documents: {documents}.")

    return words


def tokenize(text: str, remove_stopwords: bool = False, lang: str = 'en') -> list:
    """Create list of tokens based on a list of textual documents.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_text_param(text)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    stopwords = STOPWORDS[lang]
    if remove_stopwords:
        tokens = [t for t in word_tokenize(text) if len(t) > 0 and t.lower() not in stopwords]
    else:
        tokens = [t for t in word_tokenize(text) if len(t) > 0]
    return tokens


def avg_syllable_per_word(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Count average number of syllables per word based on a list of textual documents.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords=remove_stopwords)
    syl_count = syllable_count(words, lang)
    return syl_count / len(words)


def sentence_count(documents: list) -> int:
    """Count number of sentences in a list of textual documents

    Keyword arguments:
    documents -- the list of textual documents.
    """
    __check_documents_param(documents)
    cnt = 0
    for doc in documents:
        sent_cnt = len([t for t in re.split(r'[.!?\.]+', doc) if len(t) > 0])
        if len(doc) > 0 and sent_cnt == 0:
            cnt += 1
        else:
            cnt += sent_cnt
    return cnt


def get_sentences(documents: list) -> list:
    """Convert a list of textual documents into a list of sentences.

    Keyword arguments:
    documents -- the list of textual documents.
    """
    __check_documents_param(documents)

    sentences = list()
    for doc in documents:
        sentences += [t for t in re.split(r'[.!?\.]+', doc) if len(t) > 0]
    return sentences


def avg_word_length(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Count average word length based on a list of textual documents.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords=remove_stopwords)
    char_cnt = char_count(documents)
    return char_cnt/len(words)


def avg_sentence_length(documents: list, ignore_spaces: bool = True) -> float:
    """Count average sentence length in a list of documents.

    Keyword arguments:
    documents -- list of textual documents.
    ignore_spaces -- boolean flag that shows if we should ignore spaces
    """
    __check_documents_param(documents)
    __check_bool_param(ignore_spaces)

    return char_count(documents, ignore_spaces)/sentence_count(documents)


def get_ngrams(
        documents: list,
        n: int = 1,
        lang: str = 'en',
        remove_stopwords: bool = True,
        output_count: bool = True
) -> list:
    """Retrieve a list of ngrams based on a list of textual documents.

    Keyword arguments:
    documents -- the list of textual documents
    n -- the "n" parameter in "ngram"
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    output_count -- boolean flag that shows if the function should return ngram occurrences
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)
    __check_bool_param(output_count)
    if n < 1:
        raise ValueError(f"The n parameter should be more than 0. Got: {n}")

    words = get_words(documents, lang, remove_stopwords=remove_stopwords)
    ngrams = list()
    for i in range(0, len(words), 1):
        if i + n <= len(words):
            ngrams.append(' '.join(word for word in words[i:i+n]).lower())
    if output_count:
        return collections.Counter(ngrams).most_common()
    return ngrams


def get_lexical_items(documents: list, remove_stopwords: bool = False, lang: str = 'en') -> list:
    """Retrieve a list of lexical items (types) based on a list of textual documents.
    Lexical items (types) are: nouns, adjectives, verbs, adverbs

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    morph = pymorphy2.MorphAnalyzer()

    lex_items = list()
    nltk_tags = [
        'NN', 'NNS', 'NNP', 'NNPS',
        'JJ', 'JJR', 'JJS',
        'VB', 'VBD', 'VBG', 'VBN', 'VBP', 'VBZ',
        'RB', 'RBR', 'RBS'
    ]
    morphy_tags = [
        'NOUN', 'ADJF', 'ADJS', 'VERB', 'INFN', 'ADVB'
    ]

    for doc in documents:
        tokens = tokenize(text=doc, remove_stopwords=remove_stopwords, lang=lang)
        if lang == 'ru':
            tags = [morph.parse(token)[0].tag.POS for token in tokens]

            for i in range(len(tags)):
                if tags[i] in morphy_tags:
                    lex_items.append((tokens[i], tags[i]))
        elif lang == 'en':
            tags = pos_tag(tokens)

            for i in range(len(tags)):
                if tags[i][1] in nltk_tags:
                    lex_items.append((tokens[i], tags[i][1]))
    return lex_items


def words_per_sentence(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> list:
    """Calculate average number of words in a sentence based on a list of documents.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the words
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords)
    return len(words)/sentence_count(documents)



from linguaf.descriptive_statistics import get_words, syllable_count, avg_word_length, \
    number_of_n_syllable_words, sentence_count, words_per_sentence
from linguaf import __check_bool_param, __check_documents_param, __check_lang_param, \
    __check_text_param, __check_words_param


def flesch_reading_ease(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Flesch Reading Ease score based on a list of documents.
    100 -- easy to read. 0 -- hard to read.
    See Wikipedia article: https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests
    Publication: Flesch 1948

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords)
    asl = words_per_sentence(documents, lang, remove_stopwords)
    syl_total = syllable_count(words, lang)

    if lang == 'en':
        return 206.835 - 1.015*asl - 84.6*(syl_total/len(words))
    elif lang == 'ru':
        return 206.835 - 1.3*asl - 60.1*(syl_total/len(words))  # coefficients for russian


def flesch_kincaid_grade(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Flesch-Kincaid grade level that corresponds to a U.S. grade level.
    The higher the grade the more difficult the text.
    See Wikipedia article: https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests
    Publication: Flesch and Kincaid 1975

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords)
    asl = words_per_sentence(documents)
    syl_total = syllable_count(words, lang)

    if lang == 'en':
        return 0.39*asl + 11.8*(syl_total/len(words)) - 15.59
    elif lang == 'ru':
        return 0.5*asl + 8.4*(syl_total/len(words)) - 15.59  # coefficients for russian


def automated_readability_index(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates automated readability index based on a list of textual documents.
    The more the index the harder the text.
    The score corresponds to a grade level (similar to Flesh-Kincaid grade).
    See Wikipedia article: https://en.wikipedia.org/wiki/Automated_readability_index
    Publication: Senter and Smith 1967

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    asl = words_per_sentence(documents, lang, remove_stopwords)
    awl = avg_word_length(documents, lang, remove_stopwords)

    return 0.5*asl + 4.71*awl - 21.43


def automated_readability_index_simple(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates simplified automated readability index based on a list of textual documents.
    The more the index the harder the text.
    See Wikipedia article: https://en.wikipedia.org/wiki/Automated_readability_index
    Publication: Senter and Smith 1967

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    asl = words_per_sentence(documents, lang, remove_stopwords)
    awl = avg_word_length(documents, lang, remove_stopwords)

    return asl + 9.0*awl


def coleman_readability(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Coleman's Readability score based on a list of textual documents.
    Publication: Coleman, E.B. (1971)

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents, lang, remove_stopwords)
    nws_one = number_of_n_syllable_words(words, lang, (1, 2))

    return 1.29*(100*nws_one/len(words)) - 38.45


def easy_listening(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Easy Listening score based on a list of textual documents.
    Publication: Fang 1966

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    nst = sentence_count(documents)
    words = get_words(documents, lang, remove_stopwords)
    nws_more_two = number_of_n_syllable_words(words, lang, (2, 15))

    return nws_more_two/nst



import collections
import math
from linguaf.descriptive_statistics import get_words, get_lexical_items
from linguaf import __check_bool_param, __check_documents_param, __check_lang_param


def lexical_density(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates lexical density based on a list of documents.
    Lexical density is the ratio between number of lexical items and number of words in total.
    See Wikipedia article: https://en.wikipedia.org/wiki/Lexical_density

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents=documents, lang=lang, remove_stopwords=remove_stopwords)
    lex_items = get_lexical_items(documents=documents, remove_stopwords=remove_stopwords, lang=lang)
    return len(lex_items)/len(words)*100


def type_token_ratio(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Type-Token Ratio based on a list of documents.
    Types -- are unique words, Tokens (in this context) -- are words in total.
    See Wikipedia article: https://de.wikipedia.org/wiki/Type-Token-Relation

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents=documents, lang=lang, remove_stopwords=remove_stopwords)
    num_unq = len(collections.Counter(words).keys())
    return num_unq/len(words)*100


def log_type_token_ratio(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Log Type-Token Ratio (Herdan's Constant) based on a list of documents.
    Types -- are unique words, Tokens (in this context) -- are words in total.
    Publicaiton: Herdan, 1960, as cited in Tweedie & Baayen, 1998

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents=documents, lang=lang, remove_stopwords=remove_stopwords)
    num_unq = len(collections.Counter(words).keys())
    return math.log(num_unq)/math.log(len(words))*100


def summer_index(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Summer's Index based on a list of documents.
    The index is the same as Double Log Type-Token Ratio.

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents=documents, lang=lang, remove_stopwords=remove_stopwords)
    num_unq = len(collections.Counter(words).keys())
    if num_unq == 0:
        num_unq = 10**-10
    return math.log(math.log(num_unq))/math.log(math.log(len(words)))*100


def root_type_token_ratio(documents: list, lang: str = 'en', remove_stopwords: bool = False) -> float:
    """Calculates Root Type-Token Ratio based on a list of documents.
    Publication: Guiraud, 1954. Also cited in Tweedie & Baayen, 1998

    Keyword arguments:
    documents -- the list of textual documents
    lang -- language of the textual documents
    remove_stopwords -- boolean flag that shows if the function should exclude stopwords
    """
    __check_documents_param(documents)
    __check_lang_param(lang)
    __check_bool_param(remove_stopwords)

    words = get_words(documents=documents, lang=lang, remove_stopwords=remove_stopwords)
    num_unq = len(collections.Counter(words).keys())
    return num_unq/(len(words)**0.5)*100



import os
import setuptools


__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))

with open("README.md", "r") as fh:
    long_description = fh.read()


def read_requirements():
    reqs_path = os.path.join(__location__, 'requirements.txt')
    with open(reqs_path, encoding='utf8') as f:
        reqs = [line.strip() for line in f if not line.strip().startswith('#')]

    names = []
    for req in reqs:
        names.append(req)
    return {'install_requires': names}


setuptools.setup(
    name="linguaf",
    version="0.0.7",
    author="Aleksandr Perevalov",
    author_email="perevalovproduction@gmail.com",
    description="Python package for calculating famous measures in computational linguistics",
    long_description=long_description,
    license="MIT",
    long_description_content_type="text/markdown",
    url="https://github.com/Perevalov/LinguaF",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    packages=setuptools.find_packages(),
    package_data={"": ["resources/stopwords/*.json"]},
    keywords="language features computational linguistics quantitative text analysis",
    **read_requirements()
)
